grammar FSM;

options {
  language = Java;
  output=template;
}


scope slist {
    List actions; // must be defined one per semicolon
    boolean isguard;
}

@header {
import java.util.HashMap;
import org.antlr.stringtemplate.*;
}

/*------------------------------------------------------------------
 * LEXER RULES
 *------------------------------------------------------------------*/

ID : ('a'..'z' | 'A'..'Z')+;
INT: ('0'..'9')+;
NEWLINE: '\r'? '\n';
WS : (' ' | '\t')+ {skip();};

/*------------------------------------------------------------------
 * PARSER RULES
 *------------------------------------------------------------------*/

prog
scope {
  List variables;
  List statements;
  List guards;
}
@init {
  $prog::variables = new ArrayList();
  $prog::statements = new ArrayList();
  $prog::guards = new ArrayList();
}
    : 'FSM' ID '{' stat+ '}' 
        -> prog(classname={new StringTemplate($ID.text)},
                variables={$prog::variables}, 
                statements={$prog::statements},
                guards={$prog::guards})
    ;

stat: variable {$prog::variables.add($variable.st);}
    | state {$prog::statements.add($state.st);}
    | transition {$prog::statements.add($transition.st);}
    | NEWLINE
    ;

variable: type ':' ID ':' a=atom NEWLINE 
          -> variable(type={$type.st}, name={new StringTemplate($ID.text)}, value={$a.st})
        ;

state: 'state' statename=ID NEWLINE
       -> state(value={new StringTemplate($statename.text)})
    ;

transition
scope {
  String _name;
  String _from;
  String _to;
  String _event;
}
    : 'transition' tname=ID {$transition::_name=$tname.text;} 
      'from' from=ID {$transition::_from=$from.text;} 
      'to' to=ID {$transition::_to=$to.text;} ':' 
       event=ID {$transition::_event=$event.text;} 
       ttail {$prog::guards.add($ttail.st);} NEWLINE
    -> transition(tname={new StringTemplate($tname.text)},
                  from={new StringTemplate($from.text)},
                  to={new StringTemplate($to.text)},
                  event={new StringTemplate($event.text)})
    ;

ttail
scope slist;
@init {
  $slist::actions = new ArrayList();
  $slist::isguard = false;
}
     : g=guard? actionstats? -> {$slist::isguard}? 
                                guardBlock(guardcond={$g.st},
                                  t_name={$transition::_name},
                                  t_from={$transition::_from},
                                  t_to={$transition::_to},
                                  t_event={$transition::_event},
                                  actions={$slist::actions})
                             -> guardBlock(guardcond={new StringTemplate("true")},
                                  t_name={$transition::_name},
                                  t_from={$transition::_from},
                                  t_to={$transition::_to},
                                  t_event={$transition::_event},
                                  actions={$slist::actions})

     ;

guard
    : '[' 
         (a=exprboo {$slist::isguard = true;} -> {$a.st}) 
         ( '&&' b=exprboo -> andExpr(l_op={$a.st},r_op={$b.st}) 
         | '||' b=exprboo -> orExpr(l_op={$a.st},r_op={$b.st})
         )*
      ']' 
    ;

actionstats: '/' (a=action {$slist::actions.add($a.st);}) (';' b=action {$slist::actions.add($b.st);})*;

action : lhs=ID ':=' rhs=expr -> assign(lhs={$ID.text},rhs={$rhs.text})
       ;

//for expressions that evaluate to boolean
exprboo
      : (a=expr 
          ( '==' b=expr -> equals(l_op={$a.st},r_op={$b.st})
          | '<' b=expr -> lessthan(l_op={$a.st},r_op={$b.st})
          )
        | -> {$a.st}
        )
    ;

expr
    : (a=atom -> {$a.st})
      ( '+' b=atom -> add(l_op={$a.st},r_op={$b.st})
      | '-' b=atom -> subtract(l_op={$a.st},r_op={$b.st})
      )*
      ;


atom 
    : INT -> iconst(value={$INT.text})
    | ID  -> refVar(id={$ID.text})
    | '(' expr ')' -> {$expr.st}
    ;

type
    :   'int'  -> type_int()
    ;

