# This class captures the information associated with a transition

class Transition_class

  attr_reader :name, :from_state, :to_state, :event, :action_invoked

  def initialize(name)
    @name = name
    @from_state = ""
    @to_state = ""
    @guard_block = nil
    @action_block = nil
    @event = ""
    @action_invoked = ""
    [:from_state, :to_state, :event, :action_invoked].each do |method|
      Transition_class.attr_access method
    end
  end

  # define accessor methods (get and set) dyamically
  def self.attr_access(attr)
     define_method :"#{attr}" do |value|
       instance_variable_set("@#{attr}",value)
     end
     define_method :"get_#{attr}" do
       instance_variable_get("@#{attr}")
     end
  end

  # define event method that would enable the use of 'event = '
  def event=(event)
    @event = event
  end

  #def get_event
  #  @event
  #end

  # the guard code block is executed using 'call'
  def call_guard(alarm, credit)
    result = true
    if @guard_block
      result = @guard_block.call(credit)
    end
    puts " result : #{result} "
    result
  end

  # The block argument - &guardBlock - enables code block to be assigned to a variable
  def guard(&guardBlock)
    @guard_block = guardBlock
  end

  def call_action(fsm)
    if @action_block
      @action_block.call(fsm) 
    end
  end

  def action(&actionBlock)
    @action_block = actionBlock
  end

  # the load_block executes the transition code block using 'yield'
  def load_block
    yield self
  end

end
